@@ -73,13 +73,15 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: platebtn.py 62892 2009-12-15 14:51:07Z CJP $"
-__revision__ = "$Revision: 62892 $"
+__svnid__ = "$Id: platebtn.py 68296 2011-07-17 19:43:06Z CJP $"
+__revision__ = "$Revision: 68296 $"
 
 __all__ = ["PlateButton",
            "PLATE_NORMAL", "PLATE_PRESSED", "PLATE_HIGHLIGHT", 
+
            "PB_STYLE_DEFAULT", "PB_STYLE_GRADIENT", "PB_STYLE_SQUARE",
-           "PB_STYLE_NOBG", "PB_STYLE_DROPARROW",
+           "PB_STYLE_NOBG", "PB_STYLE_DROPARROW", "PB_STYLE_TOGGLE",
+
            "EVT_PLATEBTN_DROPARROW_PRESSED"]
 
 #-----------------------------------------------------------------------------#
@@ -103,9 +105,12 @@ PB_STYLE_SQUARE   = 4   # Use square corners instead of rounded
 PB_STYLE_NOBG     = 8   # Usefull on Windows to get a transparent appearance
                         # when the control is shown on a non solid background
 PB_STYLE_DROPARROW = 16 # Draw drop arrow and fire EVT_PLATEBTN_DROPRROW_PRESSED event
+PB_STYLE_TOGGLE   = 32  # Stay pressed untill clicked again
 
 #-----------------------------------------------------------------------------#
 
+# EVT_BUTTON used for normal event notification
+# EVT_TOGGLE_BUTTON used for toggle button mode notification
 PlateBtnDropArrowPressed, EVT_PLATEBTN_DROPARROW_PRESSED = wx.lib.newevent.NewEvent()
 
 #-----------------------------------------------------------------------------#
@@ -115,7 +120,7 @@ class PlateButton(wx.PyControl):
     displaying bitmaps and having an attached dropdown menu.
 
     """
-    def __init__(self, parent, id_=wx.ID_ANY, label='', bmp=None, 
+    def __init__(self, parent, id=wx.ID_ANY, label='', bmp=None, 
                  pos=wx.DefaultPosition, size=wx.DefaultSize,
                  style=PB_STYLE_DEFAULT, name=wx.ButtonNameStr):
         """Create a PlateButton
@@ -124,27 +129,29 @@ class PlateButton(wx.PyControl):
         @keyword style: Button style
 
         """
-        wx.PyControl.__init__(self, parent, id_, pos, size,
-                              wx.BORDER_NONE|wx.TRANSPARENT_WINDOW, name=name)
+        super(PlateButton, self).__init__(parent, id, pos, size,
+                                          wx.BORDER_NONE|wx.TRANSPARENT_WINDOW,
+                                          name=name)
 
         # Attributes
         self.InheritAttributes()
-        self._bmp = dict(enable=bmp)
+        self._bmp = dict(enable=None, disable=None)
         if bmp is not None:
+            assert isinstance(bmp, wx.Bitmap) and bmp.IsOk()
+            self._bmp['enable'] = bmp
             img = bmp.ConvertToImage()
             img = img.ConvertToGreyscale(.795, .073, .026) #(.634, .224, .143)
-            self._bmp['disable'] = img.ConvertToBitmap()
-        else:
-            self._bmp['disable'] = None
+            self._bmp['disable'] = wx.BitmapFromImage(img)
 
         self._menu = None
         self.SetLabel(label)
         self._style = style
         self._state = dict(pre=PLATE_NORMAL, cur=PLATE_NORMAL)
         self._color = self.__InitColors()
+        self._pressed = False
 
         # Setup Initial Size
-        self.SetInitialSize()
+        self.SetInitialSize(size)
 
         # Event Handlers
         self.Bind(wx.EVT_PAINT, lambda evt: self.__DrawButton())
@@ -153,13 +160,13 @@ class PlateButton(wx.PyControl):
         self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
 
         # Mouse Events
+        self.Bind(wx.EVT_LEFT_DCLICK, lambda evt: self._ToggleState())
         self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
         self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
-        self.Bind(wx.EVT_LEFT_DCLICK, lambda evt: self.ToggleState())
         self.Bind(wx.EVT_ENTER_WINDOW,
-                  lambda evt: self.SetState(PLATE_HIGHLIGHT))
+                  lambda evt: self._SetState(PLATE_HIGHLIGHT))
         self.Bind(wx.EVT_LEAVE_WINDOW,
-                  lambda evt: wx.CallLater(80, self.SetState, PLATE_NORMAL))
+                  lambda evt: wx.CallLater(80, self.__LeaveWindow))
 
         # Other events
         self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
@@ -178,7 +185,7 @@ class PlateButton(wx.PyControl):
 
         if bmp is not None and bmp.IsOk():
             bw, bh = bmp.GetSize()
-            ypos = (self.GetSize()[1] - bh) / 2
+            ypos = (self.GetSize()[1] - bh) // 2
             gc.DrawBitmap(bmp, 6, ypos, bmp.GetMask() != None)
             return bw + 6
         else:
@@ -236,7 +243,11 @@ class PlateButton(wx.PyControl):
 
     def __PostEvent(self):
         """Post a button event to parent of this control"""
-        bevt = wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, self.GetId())
+        if self._style & PB_STYLE_TOGGLE:
+            etype = wx.wxEVT_COMMAND_TOGGLEBUTTON_CLICKED
+        else:
+            etype = wx.wxEVT_COMMAND_BUTTON_CLICKED
+        bevt = wx.CommandEvent(etype, self.GetId())
         bevt.SetEventObject(self)
         bevt.SetString(self.GetLabel())
         self.GetEventHandler().ProcessEvent(bevt)
@@ -256,7 +267,8 @@ class PlateButton(wx.PyControl):
         # Setup
         dc.SetBrush(wx.TRANSPARENT_BRUSH)
         gc.SetBrush(wx.TRANSPARENT_BRUSH)
-        gc.SetFont(self.GetFont())
+        gc.SetFont(self.Font)
+        dc.SetFont(self.Font)
         gc.SetBackgroundMode(wx.TRANSPARENT)
 
         # The background needs some help to look transparent on
@@ -267,8 +279,11 @@ class PlateButton(wx.PyControl):
 
         # Calc Object Positions
         width, height = self.GetSize()
-        tw, th = gc.GetTextExtent(self.GetLabel())
-        txt_y = max((height - th) / 2, 1)
+        if wx.Platform == '__WXGTK__':
+            tw, th = dc.GetTextExtent(self.Label)
+        else:
+            tw, th = gc.GetTextExtent(self.Label)
+        txt_y = max((height - th) // 2, 1)
 
         if self._state['cur'] == PLATE_HIGHLIGHT:
             gc.SetTextForeground(self._color['htxt'])
@@ -285,8 +300,11 @@ class PlateButton(wx.PyControl):
 
             self.__DrawHighlight(gc, width, height)
             txt_x = self.__DrawBitmap(gc)
-            gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
-            self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
+            if wx.Platform == '__WXGTK__':
+                dc.DrawText(self.Label, txt_x + 2, txt_y)
+            else:
+                gc.DrawText(self.Label, txt_x + 2, txt_y)
+            self.__DrawDropArrow(gc, txt_x + tw + 6, (height // 2) - 2)
 
         else:
             if self.IsEnabled():
@@ -298,8 +316,11 @@ class PlateButton(wx.PyControl):
         # Draw bitmap and text
         if self._state['cur'] != PLATE_PRESSED:
             txt_x = self.__DrawBitmap(gc)
-            gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
-            self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
+            if wx.Platform == '__WXGTK__':
+                dc.DrawText(self.Label, txt_x + 2, txt_y)
+            else:
+                gc.DrawText(self.Label, txt_x + 2, txt_y)
+            self.__DrawDropArrow(gc, txt_x + tw + 6, (height // 2) - 2)
 
     def __InitColors(self):
         """Initialize the default colors"""
@@ -311,6 +332,14 @@ class PlateButton(wx.PyControl):
                       htxt=BestLabelColour(self.GetForegroundColour()))
         return colors
 
+    def __LeaveWindow(self):
+        """Handle updating the buttons state when the mouse cursor leaves"""
+        if (self._style & PB_STYLE_TOGGLE) and self._pressed:
+            self._SetState(PLATE_PRESSED) 
+        else:
+            self._SetState(PLATE_NORMAL)
+            self._pressed = False
+
     #---- End Private Member Function ----#
 
     #---- Public Member Functions ----#
@@ -345,8 +374,11 @@ class PlateButton(wx.PyControl):
         """
         width = 4
         height = 6
-        if self.GetLabel():
-            lsize = self.GetTextExtent(self.GetLabel())
+        if self.Label:
+            # NOTE: Should measure with a GraphicsContext to get right
+            #       size, but due to random segfaults on linux special
+            #       handling is done in the drawing instead...
+            lsize = self.GetFullTextExtent(self.Label)
             width += lsize[0]
             height += lsize[1]
             
@@ -420,10 +452,25 @@ class PlateButton(wx.PyControl):
         """
         return getattr(self, '_menu', None)
 
+    def GetState(self):
+        """Get the current state of the button
+        @return: int
+        @see: PLATE_NORMAL, PLATE_HIGHLIGHT, PLATE_PRESSED
+
+        """
+        return self._state['cur']
+
     def HasTransparentBackground(self):
         """Override setting of background fill"""
         return True
 
+    def IsPressed(self):
+        """Return if button is pressed (PB_STYLE_TOGGLE)
+        @return: bool
+
+        """
+        return self._pressed
+
     @property
     def LabelText(self):
         """Property for getting the label of the button"""
@@ -442,7 +489,7 @@ class PlateButton(wx.PyControl):
     def OnFocus(self, evt):
         """Set the visual focus state if need be"""
         if self._state['cur'] == PLATE_NORMAL:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
 
     def OnKeyUp(self, evt):
         """Execute a single button press action when the Return key is pressed
@@ -451,9 +498,9 @@ class PlateButton(wx.PyControl):
 
         """
         if evt.GetKeyCode() == wx.WXK_SPACE:
-            self.SetState(PLATE_PRESSED)
+            self._SetState(PLATE_PRESSED)
             self.__PostEvent()
-            wx.CallLater(100, self.SetState, PLATE_HIGHLIGHT)
+            wx.CallLater(100, self._SetState, PLATE_HIGHLIGHT)
         else:
             evt.Skip()
 
@@ -466,15 +513,18 @@ class PlateButton(wx.PyControl):
         #       handler to prevent ghost highlighting from happening when
         #       quickly changing focus and activating buttons
         if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_NORMAL)
+            self._SetState(PLATE_NORMAL)
 
     def OnLeftDown(self, evt):
         """Sets the pressed state and depending on the click position will
         show the popup menu if one has been set.
 
         """
+        if (self._style & PB_STYLE_TOGGLE):
+            self._pressed = not self._pressed
+
         pos = evt.GetPositionTuple()
-        self.SetState(PLATE_PRESSED)
+        self._SetState(PLATE_PRESSED)
         size = self.GetSizeTuple()
         if pos[0] >= size[0] - 16:
             if self._menu is not None:
@@ -497,7 +547,11 @@ class PlateButton(wx.PyControl):
             size = self.GetSizeTuple()
             if not (self._style & PB_STYLE_DROPARROW and pos[0] >= size[0] - 16):
                 self.__PostEvent()
-        self.SetState(PLATE_HIGHLIGHT)
+
+        if self._pressed:
+            self._SetState(PLATE_PRESSED)
+        else:
+            self._SetState(PLATE_HIGHLIGHT)
 
     def OnMenuClose(self, evt):
         """Refresh the control to a proper state after the menu has been
@@ -507,9 +561,9 @@ class PlateButton(wx.PyControl):
         """
         mpos = wx.GetMousePosition()
         if self.HitTest(self.ScreenToClient(mpos)) != wx.HT_WINDOW_OUTSIDE:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
         else:
-            self.SetState(PLATE_NORMAL)
+            self._SetState(PLATE_NORMAL)
         evt.Skip()
 
     #---- End Event Handlers ----#
@@ -541,7 +595,7 @@ class PlateButton(wx.PyControl):
     def SetFocus(self):
         """Set this control to have the focus"""
         if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
         super(PlateButton, self).SetFocus()
 
     def SetFont(self, font):
@@ -557,20 +611,21 @@ class PlateButton(wx.PyControl):
         super(PlateButton, self).SetLabel(label)
         self.InvalidateBestSize()
 
-    def SetLabelColor(self, normal, hlight=wx.NullColor):
+    def SetLabelColor(self, normal, hlight=wx.NullColour):
         """Set the color of the label. The optimal label color is usually
         automatically selected depending on the button color. In some
-        cases the colors that are choosen may not be optimal.
+        cases the colors that are chosen may not be optimal.
         
         The normal state must be specified, if the other two params are left
         Null they will be automatically guessed based on the normal color. To
         prevent this automatic color choices from happening either specify
         a color or None for the other params.
 
-        @param normal: Label color for normal state
+        @param normal: Label color for normal state (wx.Colour)
         @keyword hlight: Color for when mouse is hovering over
 
         """
+        assert isinstance(normal, wx.Colour), "Must supply a colour object"
         self._color['default'] = False
         self.SetForegroundColour(normal)
 
@@ -601,7 +656,7 @@ class PlateButton(wx.PyControl):
 
     def SetPressColor(self, color):
         """Set the color used for highlighting the pressed state
-        @param color: wx.Color
+        @param color: wx.Colour
         @note: also resets all text colours as necessary
 
         """
@@ -614,10 +669,11 @@ class PlateButton(wx.PyControl):
         self._color['htxt'] = BestLabelColour(self._color['hlight'])
         self.Refresh()
 
-    def SetState(self, state):
+    def _SetState(self, state):
         """Manually set the state of the button
         @param state: one of the PLATE_* values
         @note: the state may be altered by mouse actions
+        @note: Internal use only!
 
         """
         self._state['pre'] = self._state['cur']
@@ -661,11 +717,14 @@ class PlateButton(wx.PyControl):
 
             self.PopupMenu(self._menu, (xpos, size[1] + adj))
 
-    def ToggleState(self):
-        """Toggle button state"""
+    def _ToggleState(self):
+        """Toggle button state
+        @note: Internal Use Only!
+
+        """
         if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_PRESSED)
+            self._SetState(PLATE_PRESSED)
         else:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
 
     #---- End Public Member Functions ----#
